#include "FileTreeView.h"
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/exception.hpp>

WStandardItem* FileTreeView::checkedItem = NULL;

FileTreeView::FileTreeView(const std::string& path, WContainerWidget* parent)
    : WTreeView(parent), m_model(NULL)
{
    setAlternatingRowColors(true);
    setRowHeight(25);

    WApplication::instance()->useStyleSheet("icons/wt.css");
    setSelectionMode(SingleSelection);

    setPath(path);
}

void FileTreeView::itemChanged(WStandardItem* item)
{
    if (item->isChecked())
    {
        if (checkedItem)
        {
            checkedItem->setChecked(false);
        }
        checkedItem = item;
    }
    else
    {
        if (item == checkedItem)
        {
            checkedItem = NULL;
        }
    }
}

bool FileTreeView::getCheckedFile(std::string& path)
{
    if (checkedItem != NULL)
    {
        PathObject* pathObj = boost::any_cast<PathObject*>(checkedItem->data());
        boost::filesystem::path boostPath = pathObj->m_path;
        path = boostPath.string();
        return true;
    }
    return false;
}

bool FileTreeView::setPath(const std::string& path)
{
    if (m_path.compare(path) == 0)
    {
        return false;
    }
    checkedItem = NULL;
    m_path = path;

    if (m_model)
    {
        m_model->clear();
    }
    else
    {
        m_model = new WStandardItemModel(this);
    }

    ItemQueue itemQueue;

    WStandardItem* currentItem;
    boost::filesystem::path currentPath(m_path);

    currentItem = new WStandardItem(currentPath.leaf());
    PathObject* pathObj =  new PathObject(currentPath, this);

    currentItem->setData(pathObj);

    m_model->appendRow(currentItem);

    if (boost::filesystem::is_directory(currentPath) )
    {
        currentItem->setIcon("icons/yellow-folder-closed.png");
        itemQueue.push_back(currentItem);
    }
    else
    {
        currentItem->setIcon("icons/document.png");
        currentItem->setCheckable(true);
    }

    do
    {
        currentItem = itemQueue.front();
        pathObj = boost::any_cast<PathObject*>(currentItem->data(UserRole));
        currentPath = pathObj->m_path;
        itemQueue.pop_front();

        boost::filesystem::directory_iterator end_itr;
        for (boost::filesystem::directory_iterator it(currentPath); it != end_itr; it ++)
        {
            if (!boost::filesystem::is_symlink(*it))
            {
                if (boost::filesystem::is_regular_file(*it) || boost::filesystem::is_directory(*it))
                {
                    std::cout<<it->string()<<std::endl;
                    WStandardItem* sonItem =  new WStandardItem(it->leaf());
                    currentItem->appendRow(sonItem);
                    pathObj = new PathObject(*it, this);

                    sonItem->setData(pathObj);
                    if (boost::filesystem::is_directory(*it))
                    {
                        itemQueue.push_back(sonItem);
                        sonItem->setIcon("icons/yellow-folder-closed.png");
                    }
                    else
                    {
                        sonItem->setIcon("icons/document.png");
                        sonItem->setCheckable(true);
                    }
                }
            }
        }
    }
    while (!itemQueue.empty());
    m_model->itemChanged().connect(this, &FileTreeView::itemChanged);
    setModel(m_model);
    return true;
}


